

import yaml
import os
import logging

logger = logging.getLogger()

class SmartConfigFile():
    POSSIBLE_FOLDERS = ["/etc/alpibus",
                        os.path.join(os.path.expanduser("~"), ".alpibus"),
                        os.getcwd(),
                        os.getenv('ALPIBUS_DIR', "/etc/alpibus")
                        ]
    DEFAULT_CONFIGILE = "config.yml"

    def __init__(self, configfile=None):
        self.__set_configfile(configfile)
        self.load()

    def __set_configfile(self, configfile):
        if configfile is None:
            configfile = self.DEFAULT_CONFIGILE

        self.configfilepath = self.__find_configfile(configfile)

        if self.configfilepath is None:
            logger.error("No Config file found! "+ configfile)
            raise Exception("No Config file found: '%s' in %s"%(configfile, str(self.POSSIBLE_FOLDERS)))

    def __find_configfile(self, configfile):
        for folder in self.POSSIBLE_FOLDERS:
            t = os.path.join(folder, configfile)
            if os.path.isfile(t):
                logger.info("Config file found in: " + folder)
                return t
            else:
                logger.warning("Config file not found in: "+folder)
        return None

    def load(self):
        stream = file(self.configfilepath, 'r')
        self.__config = yaml.load(stream)

    def get(self, keys, default_value=None):
        dct = self.__config
        for key in keys:
            try:
                dct = dct[key]
            except KeyError:
                return default_value
        return dct









import logging
import threading
import time

from sqlalchemy import create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker

from sibus_lib.utils import float_to_datetime, datetime_now_float

logger = logging.getLogger()

CoreDBBase = declarative_base()

class CoreDatabase(threading.Thread):
    def __init__(self):
        threading.Thread.__init__(self)
        self._stopevent = threading.Event()
        self.init_db()

    def init_db(self):
        engine = create_engine("mysql+mysqldb://root:alpi@localhost:3306/car_dashboard", echo=True)
        CoreDBBase.metadata.create_all(engine)
        Session = sessionmaker()
        Session.configure(bind=engine)
        self.SessionMaker = Session
        #self.start()

    def run(self):
        session = self.SessionMaker()
        while not self._stopevent.isSet():
            for instance in self.session.query(BusElementDB).order_by(BusElementDB.last_event_date):
                logger.info("%s.%s is %s (started:%s, last:%s)"%(instance.bus_origin_host,
                                                                 instance.bus_origin_service,
                                                                 instance.status,
                                                                 float_to_datetime(instance.first_event_date),
                                                                 float_to_datetime(instance.last_event_date)))

                if (datetime_now_float() - instance.last_event_date > 60) and (instance.status <> "dead"):
                    instance.status = "dead"
                    session.add(instance)

            session.commit()
            time.sleep(5)

    def stop(self):
        self._stopevent.set( )

    def get_session(self):
        return self.SessionMaker()

    def update_buselement(self, bus_uid, host, service, event_date):
        session = self.SessionMaker()





#!/usr/bin/env python
# -*- coding: utf-8 -*-
import datetime as dt
import random

def nice_float(value):
    return float("%.6f" % float(value))

def parse_num(s):
    try:
        return int(s)
    except ValueError:
        try:
            return nice_float(s)
        except ValueError:
            return s

def datetime_now():
    return dt.datetime.now().strftime("%Y-%m-%dT%H:%M:%S.%f")

def random_value(value, percent_range=30):
    t = (random.random() - 0.5)*(percent_range/100.0)
    return value * (1+t)

def datetime_to_float(d):
    epoch = dt.datetime.utcfromtimestamp(0)
    total_seconds =  (d - epoch).total_seconds()
    # total_seconds will be in decimals (millisecond precision)
    return total_seconds

def float_to_datetime(fl):
    return dt.datetime.fromtimestamp(fl)

def datetime_now_float():
    return datetime_to_float(dt.datetime.utcnow())

def sibus_install():
    print "test"



# -*- coding: utf-8 -*-

import logging
import os

from logging.handlers import RotatingFileHandler

LOG_DIRECTORY = "/tmp/car.dashboard"

def mylogger(logger_name):
    # création de l'objet logger qui va nous servir à écrire dans les logs
    logger = logging.getLogger()
    # on met le niveau du logger à DEBUG, comme ça il écrit tout
    logger.setLevel(logging.DEBUG)

    # création d'un formateur qui va ajouter le temps, le niveau
    # de chaque message quand on écrira un message dans le log
    formatter = logging.Formatter('%(asctime)s|%(levelname)08s | %(message)s')
    stream_formatter = logging.Formatter('%(levelname)08s | %(message)s')
    # création d'un handler qui va rediriger une écriture du log vers
    # un fichier en mode 'append', avec 1 backup et une taille max de 1Mo

    if not os.path.isdir(LOG_DIRECTORY):
        print "Creating log directory: ", LOG_DIRECTORY
        os.makedirs(LOG_DIRECTORY)

    log_file = os.path.join(LOG_DIRECTORY, logger_name+'.log')
    print "Log file: ", log_file
    file_handler = RotatingFileHandler(log_file, 'a', 1000000, 1)
    # on lui met le niveau sur DEBUG, on lui dit qu'il doit utiliser le formateur
    # créé précédement et on ajoute ce handler au logger
    file_handler.setLevel(logging.INFO)
    file_handler.setFormatter(formatter)
    logger.addHandler(file_handler)

    # création d'un second handler qui va rediriger chaque écriture de log
    # sur la console
    steam_handler = logging.StreamHandler()
    steam_handler.setLevel(logging.DEBUG)
    steam_handler.setFormatter(stream_formatter)
    logger.addHandler(steam_handler)

    return logger


# !/usr/bin/env python
# -*- coding: utf-8 -*-

"""
    Ce module proclame la bonne parole de sieurs Sam et Max. Puissent-t-ils
    retrouver une totale liberté de pensée cosmique vers un nouvel age
    reminiscent.
"""

__version__ = "0.0.5"

from sibus_lib.DefaultLogger import mylogger
from sibus_lib.MessagingCore import BusElement, BusCore, MessageObject
from sibus_lib.SmartConfig import SmartConfigFile
from sibus_lib.CoreDatabase import CoreDatabase, CoreDBBase




#!/usr/bin/env python
# -*- coding: utf-8 -*-
import base64
import datetime as dt
import json
import logging
import re
import socket
import threading
import time
import uuid

import zmq
from sibus_lib.SmartConfig import SmartConfigFile
from sibus_lib.utils import float_to_datetime, datetime_now_float
from marshmallow import Schema, fields, post_load, ValidationError

logger = logging.getLogger()

DATAQUEUE_NAME = "dataflow"
ADMINQUEUE_NAME = "adminflow"


###############################################################################################
###############################################################################################

class MessageSchema(Schema):
    uid = fields.Str()
    origin_host = fields.Str()
    origin_service = fields.Str()
    origin_uid = fields.Str()
    date_creation = fields.Float()
    topic = fields.Str()
    b64data = fields.Str()

    @post_load
    def make_object(self, data):
        msg = MessageObject()
        msg.date_creation = data["date_creation"]
        msg.origin_host = data["origin_host"]
        msg.origin_service = data["origin_service"]
        msg.origin_uid = data["origin_uid"]
        msg.uid = data["uid"]
        msg.topic = data["topic"]
        msg.b64data = data["b64data"]
        return msg

    def toObject(self, json_string):
        message = self.loads(json_string)
        return message.data

###############################################################################################
###############################################################################################

class MessageObject:

    def __init__(self, data=None,
                    topic="*"
                 ):
        # type: (object, object) -> object
        self.uid = str(uuid.uuid1())
        self.origin_host = socket.getfqdn()
        self.origin_service = None
        self.origin_uid = None
        self.date_creation = datetime_now_float()
        self.topic = topic
        self.set_data(data)

    def __repr__(self):
        return "<BusMessage> origin:%s.%s; topic:%s; date:%s; data:%s"%(self.origin_host, self.origin_service, self.topic,
                                                                            str(float_to_datetime(self.date_creation)),
                                                                            self.get_data())

    def set_data(self, data):
        if data is None:
            self.b64data = ""
            return
        if type(data) <> dict:
            logger.error("Data in message bus must be provided as 'dict' !!")
            self.b64data = ""
            return
        self.b64data = base64.b64encode(json.dumps(data))

    def get_data(self):
        if len(self.b64data) == 0:
            return None
        return json.loads(base64.b64decode(self.b64data))

    def toJson(self):
        schema = MessageSchema()
        json_result = schema.dumps(self)
        return json_result.data


###############################################################################################
###############################################################################################

cfg_data = SmartConfigFile()
print cfg_data.configfilepath

SERVER_IP = cfg_data.get(["buscore","host",],"127.0.0.1")
logger.info("Bus Core Config IP: "+SERVER_IP)

PUBLISH_PORT = cfg_data.get(["buscore","internal_port",],1111)
logger.info("Bus Core internal queue port: "+str(PUBLISH_PORT))

MESSAGE_PORT = cfg_data.get(["buscore","port",],1981)
logger.info("Bus Core listening queue port: "+str(MESSAGE_PORT))

LISTEN_INTF_SRV = "tcp://*:%s"%PUBLISH_PORT
PUBLISH_INTF_SRV = "tcp://*:%s"%MESSAGE_PORT

LISTEN_INTF_CLT = "tcp://%s:%s"%(SERVER_IP,MESSAGE_PORT)
PUBLISH_INTF_CLT = "tcp://%s:%s"%(SERVER_IP,PUBLISH_PORT)

class BusCore(threading.Thread):
    def __init__(self):
        threading.Thread.__init__(self)
        self._stopevent = threading.Event()

        # Initialize the ZeroMQ context
        self.context = zmq.Context()
        self._init_reception_queue()
        self._init_emission_queue()


    def _init_reception_queue(self):
        # Configure ZeroMQ to receive messages
        self.zmq_recv = self.context.socket(zmq.SUB)
        # The communication is made on socket 1112
        self.zmq_recv.bind(LISTEN_INTF_SRV)
        self.zmq_recv.setsockopt(zmq.SUBSCRIBE, '')  # subscribe to everything
        logger.info("Bus Core Receiving messages on "+LISTEN_INTF_SRV)

    def _init_emission_queue(self):
        # Configure ZeroMQ to send messages
        self.zmq_send = self.context.socket(zmq.PUB)
        # The communication is made on socket 1111
        self.zmq_send.bind(PUBLISH_INTF_SRV)
        logger.info("Bus Core Sending messages on "+PUBLISH_INTF_SRV)

    def publish(self, msg):
        """self.channel.basic_publish(exchange='',
                              routing_key=DATAQUEUE_NAME,
                              body=msg.msg())"""
        json_result = msg.toJson()

        logger.info("Core Bus publishing message : %s"%str(msg))

        try:
            self.zmq_send.send(json_result)
        except zmq.ZMQError as err:
            logger.info("Error while trying to publish the message : %s"%str(err))
            return False

        return True

    def run(self):
        logger.info("Bus core start listening messages")
        while not self._stopevent.isSet():
            # Command to wait for an incoming message
            try:
                # Capture the message and store it
                body = self.zmq_recv.recv(zmq.NOBLOCK)
            except zmq.ZMQError as err:
                #logger.error("ZMQ Error : %s" % str(err))
                time.sleep(0.01)
                continue

            schema = MessageSchema(strict=True)
            try:
                message = schema.toObject(body)
            except ValidationError as err:
                logger.error("Invalid message format: %s, dropping it" % str(body))
                logger.error("    ERRORS: %s" % str(err.messages))
                return

            logger.debug("Bus Core received a message : %s" % str(message))
            #self.client_list.client_message(message)
            """self._db.update_buselement(bus_uid=message.origin_uid,
                                       host=message.origin_host,
                                       service=message.origin_service,
                                       event_date=message.date_creation)
            """
            self.publish(message)

        logger.info("Bus core stopped listening message.")

    def stop(self):
        self._stopevent.set( )
        self.zmq_send.close()
        self.zmq_recv.close()
        self.context.term()
        logger.info("Bus core cleaned and terminated. Thank you !")

##############################################################################################""

class BusElement(threading.Thread):

    def __init__(self, service_name,
                 callback=None,
                 filter=None,
                 ignore_my_msg=True):
        threading.Thread.__init__(self)
        self._stopevent = threading.Event()
        # Initialize the ZeroMQ context
        self.context = zmq.Context()
        self._init_reception_queue()
        self._init_emission_queue()
        self._ignore_me = ignore_my_msg

        self.registered_topics = []

        self.service_name = service_name
        self.bus_uid = str(uuid.uuid1())
        self.set_callback(callback=callback)

    def register_topic(self, topic_pattern):
        if topic_pattern == "*":
            regex = topic_pattern
        else:
            regex = re.compile(topic_pattern)
        self.registered_topics.append(regex)

    def set_callback(self, callback):
        self._callback = callback

    def _init_reception_queue(self):
        # Configure ZeroMQ to receive messages
        self.zmq_recv = self.context.socket(zmq.SUB)
        # The communication is made on socket 1112
        logger.info("Receiving messages on " + LISTEN_INTF_CLT)
        self.zmq_recv.connect(LISTEN_INTF_CLT)
        self.zmq_recv.setsockopt(zmq.SUBSCRIBE, '')  # subscribe to everything

    def _init_emission_queue(self):
        # Configure ZeroMQ to send messages
        self.zmq_send = self.context.socket(zmq.PUB)
        logger.info("Sending messages on " + PUBLISH_INTF_CLT)
        # The communication is made on socket 1111
        self.zmq_send.connect(PUBLISH_INTF_CLT)


    def __internal_data_cb(self, body):
        schema = MessageSchema(strict=True)
        try:
            message = schema.toObject(body)
        except ValidationError as err:
            logger.error("Invalid message format: %s, dropping it" % str(body))
            logger.error("    ERRORS: %s" % str(err.messages))
            return
        pass
        if (message.origin_uid == self.bus_uid) and self._ignore_me is True:
            #ignore message published by me !!
            return

        for topic in self.registered_topics:
            if topic == "*" or topic.match(message.topic) is not None:
                if self._callback is not None:
                    logger.info("Message triggered: %s" % str(message))
                    self._callback(message)

    def publish(self, msg):
        """self.channel.basic_publish(exchange='',
                              routing_key=DATAQUEUE_NAME,
                              body=msg.msg())"""
        msg.origin_uid = self.bus_uid
        msg.origin_service = self.service_name
        json_result = msg.toJson()

        logger.info("Publishing message : %s" % str(msg))

        try:
            self.zmq_send.send(json_result)
        except zmq.ZMQError as err:
            logger.info("ZMQ error trying to send message: %s"%str(err))

    def run(self):
        logger.info("Bus client start listening messages on bus")
        self.publish(MessageObject(topic="admin.start"))
        dt_start = dt.datetime.now()
        while not self._stopevent.isSet():
            # Command to wait for an incoming message
            try:
                # Capture the message and store it
                body = self.zmq_recv.recv(zmq.NOBLOCK)
                #logger.debug("Message received : %s"%str(body))
            except zmq.ZMQError as err:
                #logger.error("ZMQ error message : %s" % str(err))
                time.sleep(0.01)
                dt_now = dt.datetime.now()
                if (dt_now-dt_start).seconds > 30:
                    self.publish(MessageObject(topic="admin.heartbeat"))
                    dt_start = dt_now
                continue

            self.__internal_data_cb(body)
        logger.info("Bus client stopped listening messages on bus")

    def stop(self):
        self.publish(MessageObject(topic="admin.terminated"))
        time.sleep(0.5)
        self._stopevent.set( )
        self.zmq_send.close()
        self.zmq_recv.close()
        self.context.term()
        logger.info("Bus client cleaned and terminated. Thank you !")




#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

# notez qu'on import la lib
# donc assurez-vous que l'importe n'a pas d'effet de bord
import sibus_lib

# Ceci n'est qu'un appel de fonction. Mais il est trèèèèèèèèèèès long
# et il comporte beaucoup de paramètres
setup(

    # le nom de votre bibliothèque, tel qu'il apparaitre sur pypi
    name='sibus_lib',

    # la version du code
    version=sibus_lib.__version__,

    # Liste les packages à insérer dans la distribution
    # plutôt que de le faire à la main, on utilise la foncton
    # find_packages() de setuptools qui va cherche tous les packages
    # python recursivement dans le dossier courant.
    # C'est pour cette raison que l'on a tout mis dans un seul dossier:
    # on peut ainsi utiliser cette fonction facilement
    packages=find_packages(),

    # votre pti nom
    author="PicusAlex",

    # Votre email, sachant qu'il sera publique visible, avec tous les risques
    # que ça implique.
    author_email="picusalex@yahoo.fr",

    # Une description courte
    description="Library for SiBus elements",

    # Une description longue, sera affichée pour présenter la lib
    # Généralement on dump le README ici
    long_description=open('README.md').read(),

    # Vous pouvez rajouter une liste de dépendances pour votre lib
    # et même préciser une version. A l'installation, Python essayera de
    # les télécharger et les installer.
    #
    # Ex: ["gunicorn", "docutils >= 0.3", "lxml==0.5a7"]
    #
    # Dans notre cas on en a pas besoin, donc je le commente, mais je le
    # laisse pour que vous sachiez que ça existe car c'est très utile.
    install_requires= ["zmq", "PyYaml", "marshmallow", "sqlalchemy", "MySQL-python", "python-dateutil"],

    # Active la prise en compte du fichier MANIFEST.in
    include_package_data=True,

    # Une url qui pointe vers la page officielle de votre lib
    url='https://github.com/picusalex/sibus_lib',

    # Il est d'usage de mettre quelques metadata à propos de sa lib
    # Pour que les robots puissent facilement la classer.
    # La liste des marqueurs autorisées est longue:
    # https://pypi.python.org/pypi?%3Aaction=list_classifiers.
    #
    # Il n'y a pas vraiment de règle pour le contenu. Chacun fait un peu
    # comme il le sent. Il y en a qui ne mettent rien.
    classifiers=[
        "Programming Language :: Python",
        "Development Status :: 1 - Planning",
        "License :: OSI Approved",
        "Natural Language :: French",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2.7",
        "Topic :: Communications",
    ],

    # C'est un système de plugin, mais on s'en sert presque exclusivement
    # Pour créer des commandes, comme "django-admin".
    # Par exemple, si on veut créer la fabuleuse commande "proclame-sm", on
    # va faire pointer ce nom vers la fonction proclamer(). La commande sera
    # créé automatiquement.
    # La syntaxe est "nom-de-commande-a-creer = package.module:fonction".
    entry_points={
        'console_scripts': [
            'sibus-install = sibus_lib.utils:sibus_install',
        ],
    },

    # A fournir uniquement si votre licence n'est pas listée dans "classifiers"
    # ce qui est notre cas
    license="WTFPL",

    # Il y a encore une chiée de paramètres possibles, mais avec ça vous
    # couvrez 90% des besoins

)


#!/usr/bin/env python
# -*- coding: iso-8859-1 -*-

import argparse
import base64
import urllib
import urllib2
import subprocess
import os
import sys
import logging
import datetime as dt

__author__ = 'Alex'

logger = logging.getLogger()

class TextToSpeech:
    TTS_URL = 'http://api.voicerss.org/?'
    DOC_URL = "http://www.voicerss.org/api/documentation.aspx"
    APIKEY = "52a168f7eeb74d2ba633eb074453f66c"

    FILEDIR = "/tmp/voiceRSS/tts"

    def __init__(self, text, lang='fr-fr', debug=False):
        if not os.path.isdir(self.FILEDIR):
            os.makedirs(self.FILEDIR)

        self.debug = debug
        self.lang = lang
        self.set_text(text)

    def set_text(self, text):
        if not text:
            raise Exception('No text to speak')

        text = text.replace('\n', '').strip()

        logger.info("Set TTS text: '%s'" % text)

        if not len(text):
            raise Exception('Empty text !')

        self._audio_data = None
        self._text = text
        self._filepath = self.filepath()

    def filepath(self):
        tmp = self._text.lower()
        # print tmp
        chars = "\\`*{}[]()>#+-.!$"
        for c in chars:
            if c in tmp:
                tmp = tmp.replace(c, "")
        # print tmp
        tmp = tmp.replace(" ", "_")
        # print tmp
        tmp = os.path.join(self.FILEDIR, "%s.mp3" % tmp)
        # print tmp
        return tmp

    def _generate_tts_audio(self):
        params = {'key': self.APIKEY,
                  'src': self._text,
                  'hl': self.lang,
                  'c': "MP3",
                  'f': "44khz_16bit_mono",
                  'r': 1
                  }

        headers = {
            'User-Agent': "Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/46.0.2490.86 Safari/537.36",
            }

        data = urllib.urlencode(params)
        logger.info("Calling TTS API: '%s'" % self.TTS_URL)
        logger.info("  headers: %s" % headers)
        logger.info("  params: %s" % params)
        req = urllib2.Request(self.TTS_URL, data, headers)
        try:
            handle = urllib2.urlopen(req)
        except urllib2.URLError, e:
            raise Exception('ERROR: TTS server not reachable ! (%s)' % e.reason)
        except urllib2.HTTPError, e:
            raise Exception('ERROR: TTS server fails ! (%s)' % e.code)
        else:
            audio_data = handle.read()
            if not audio_data.startswith("ID3"):
                raise Exception('ERROR: TTS returns invalid data: %s' % audio_data)
            print " > returns %d octets" % len(audio_data)
        return audio_data

    def generateMP3(self, filepath=None, force=False):
        if filepath is None: filepath = self._filepath
        """ Do the Web request and save to `savefile` """
        if not os.path.isfile(filepath) or force == True:
            logger.info("Saving speech in '%s'" % filepath)
            with open(filepath, 'wb') as f:
                audio_data = self._generate_tts_audio()
                f.write(audio_data)
        else:
            logger.info("Speech already exists: %s" % filepath)
        return filepath

    def get_mp3_data(self):
        raw_data = ""
        with open(self._filepath, 'rb') as f:
            raw_data = f.read()
        return base64.b64encode(raw_data)

class AudioPlayer:
    FILEDIR = "/tmp/voiceRSS/play"

    def __init__(self):
        if not os.path.isdir(self.FILEDIR):
            os.makedirs(self.FILEDIR)

    def create_mp3file(self, content, filename=None):
        if filename is None: filename = dt.datetime.now().isoformat()+".mp3"
        filepath = os.path.join(self.FILEDIR, filename)
        logger.info("Saving MP3 file in '%s'" % filepath)
        raw_data = base64.b64decode(content)
        with open(filepath, 'wb') as f:
            f.write(raw_data)
        return filepath

    def playfile(self, filepath):
        if not os.path.isfile(filepath):
            logger.error("Audio Player: file '%s' does not exist"%filepath)
            return

        try:
            logger.info("Playing file: %s" % filepath)
            p = subprocess.Popen(["/usr/bin/mplayer", filepath], stderr=subprocess.PIPE)
            rc = p.poll()
            while rc is None:
                output = p.stderr.readline()
                if output != "":
                    print output.strip()
                rc = p.poll()
            if (rc != 0):
                raise Exception("ERROR: Process exits with code: %d" % rc)
            else:
                logger.info("Play TTS ends correctly")
        except subprocess.CalledProcessError as e:
            raise Exception(e)


######################################################################################
if __name__ == '__main__':
    text = " ".join(sys.argv[1:])
    tts = TextToSpeech(text)
    tts.play()

    exit(0)
